/********************************************************
  BCL Example Problems
  ====================

  file xbdlvriis.c
  ````````````````
  Transportation problem (infeasible data).
  Retrieving and printing IIS.

  (c) 2008-2024 Fair Isaac Corporation
      author: S.Heipcke, 2005, rev. Mar. 2011
********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "xprb.h"

#define NSupp 10                        /* Number of suppliers */
#define NCust 7                         /* Number of customers */
#define MaxArcs 100                     /* Max. num. of non-zero cost values */

#define VANFILE XPRBDATAPATH "/delivery/ifvan.dat"    /* Van data file */
#define COSTFILE XPRBDATAPATH "/delivery/cost.dat"    /* Cost data file */

/****DATA****/
/* Supplier:      London  Luton  B'ham Bristl  Derby Stckpt   York */
double SUPPLY[] = {140.0, 200.0,  50.0,  10.0, 400.0, 200.0,  20.0,
/* Supplier: Derby  Soton Scnthp */
             90.0,  30.0,  12.0};
/* Customer:       London Livpol Doncst   York   Hull  Manchr Shffld */
double DEMAND[] = {1230.3, 560.4, 117.1, 592.8, 310.0, 1247.0,  86.0};

double COST[NSupp][NCust];        /* Cost per supplier-customer pair */

double IFVAN[NSupp][NCust];       /* Non-zero if route uses vans instead
                                     of lorries */
double VANCAP=40.0;               /* Capacity on routes that use vans */

/***********************************************************************/

void moddelivery(XPRBprob prob)
{
 XPRBctr ctr;
 int s,c,i;
 XPRBvar x[NSupp][NCust];
 XPRBctr *iisctr;
 XPRBvar *iisvar;
 int numv, numc, numiis, ct;

/****VARIABLES****/
 for(s=0;s<NSupp;s++)
  for(c=0; c<NCust; c++)
   x[s][c]=XPRBnewvar(prob,XPRB_PL,XPRBnewname("x_s%d",s), 0, XPRB_INFINITY);

/****OBJECTIVE****/
 ctr = XPRBnewctr(prob,"OBJ",XPRB_N);
 for(s=0;s<NSupp;s++)             /* Objective: Minimize total cost */
  for(c=0; c<NCust; c++)
   XPRBaddterm(ctr, x[s][c], COST[s][c]);
 XPRBsetobj(prob,ctr);            /* Set objective function */

/****CONSTRAINTS****/
 for(c=0; c<4; c++)               /* Satisfy demand of each customer */
 {
  ctr = XPRBnewctr(prob,"Demand", XPRB_G);
  for(s=0;s<5;s++) XPRBaddterm(ctr, x[s][c], 1);
  XPRBaddterm(ctr, NULL, DEMAND[c]);
 }

 for(c=4; c<NCust; c++)           /* Satisfy demand of each customer */
 {
  ctr = XPRBnewctr(prob,"Demand", XPRB_G);
  for(s=5;s<NSupp;s++) XPRBaddterm(ctr, x[s][c], 1);
  XPRBaddterm(ctr, NULL, DEMAND[c]);
 }

 for(s=0;s<5;s++)                 /* Keep within supply at each supplier*/
 {
  ctr = XPRBnewctr(prob,"Supply",XPRB_L);
  for(c=0; c<4; c++)
   XPRBaddterm(ctr, x[s][c], 1);
  XPRBaddterm(ctr, NULL, SUPPLY[s]);
 }

 for(s=5;s<NSupp;s++)             /* Keep within supply at each supplier*/
 {
  ctr = XPRBnewctr(prob,"Supply",XPRB_L);
  for(c=4; c<NCust; c++)
   XPRBaddterm(ctr, x[s][c], 1);
  XPRBaddterm(ctr, NULL, SUPPLY[s]);
 }

/****BOUNDS****/
 for(s=0;s<NSupp;s++)
  for(c=0; c<NCust; c++)
   if(IFVAN[s][c]!=0) XPRBsetub(x[s][c], VANCAP);

/****SOLVING + OUTPUT****/
 XPRBsetsense(prob,XPRB_MINIM);   /* Set objective sense to minimization */

 XPRBlpoptimize(prob,"");         /* Solve the LP-problem */

 printf("LP status: %d\n", XPRBgetlpstat(prob));
 if (XPRBgetlpstat(prob)==XPRB_LP_OPTIMAL)
 {
  printf("Objective: %g\n",XPRBgetobjval(prob));   /* Get objective value */
 }
 else if (XPRBgetlpstat(prob)==XPRB_LP_INFEAS)     /* Get the IIS */
 {
  numiis=XPRBgetnumiis(prob);     /* Get the number of independent IIS */
  printf("Number of IIS: %d\n", numiis);
  for(s=1;s<=numiis;s++)
  {
   XPRBgetiis(prob, &iisvar, &numv, &iisctr, &numc, s);
   printf("IIS %d:  %d variables, %d constraints\n", s, numv, numc);
   if (numv>0)
   {                              /* Print all variables in the IIS */
    printf("        Variables: ");
    for(i=0;i<numv;i++) printf("%s ", XPRBgetvarname(iisvar[i]));
    printf("\n");
    XPRBfreemem(iisvar);          /* Free the array of variables */
   }
   if (numc>0)
   {                              /* Print all constraints in the IIS */
    printf("        Constraints: ");
    for(i=0;i<numc;i++) printf("%s ", XPRBgetctrname(iisctr[i]));
    printf("\n");
    XPRBfreemem(iisctr);          /* Free the array of constraints */
   }
  }

/* Alternative way of enumerating IIS: */
  XPRBgetiis(prob, NULL, &numv, NULL, &numc, 0);
  printf("IIS approximation:  %d variables, %d constraints with non-zero reduced cost/dual values\n", numv, numc);
  ct=0;
  while(numv+numc>0)
  {
   if(ct>0)
    printf("IIS %d:  %d variables, %d constraints\n", ct, numv, numc);
   ct++;
   XPRBgetiis(prob, NULL, &numv, NULL, &numc, ct);
  }

/* Retrieve variables only */
  for(s=1;s<=numiis;s++)
  {
   XPRBgetiis(prob, &iisvar, &numv, NULL, NULL, s);
   printf("IIS %d:  %d variables ( ", s, numv);
   if (numv>0)
   {
    for(i=0;i<numv;i++) printf("%s ", XPRBgetvarname(iisvar[i]));
    XPRBfreemem(iisvar);          /* Free the array of variables */
   }
   printf(")\n");
  }

/* Retrieve constraints only */
  for(s=1;s<=numiis;s++)
  {
   XPRBgetiis(prob, NULL, NULL, &iisctr, &numc, s);
   printf("IIS %d:  %d constraints ( ", s, numc);
   if (numc>0)
   {
    for(i=0;i<numc;i++) printf("%s ", XPRBgetctrname(iisctr[i]));
    XPRBfreemem(iisctr);          /* Free the array of constraints */
   }
   printf(")\n");
  }

 }

}

/***********************************************************************/

    /**** Read data from files ****/
void readdata(void)
{
 FILE *datafile;
 int s,c;

        /* Initialize data tables to 0 */
 for(s=0;s<NSupp;s++)
  for(c=0; c<NCust; c++)
  {
   COST[s][c] = 0;
   IFVAN[s][c] = 0;
  }
        /* Read the demand data file */
 datafile=fopen(COSTFILE,"r");
 for(s=0;s<NSupp;s++)
  XPRBreadarrlinecb(XPRB_FGETS, datafile, 99, "g,", COST[s], NCust);
 fclose(datafile);

        /* Read the van data file */
 datafile=fopen(VANFILE,"r");
 for(s=0;s<NSupp;s++)
  XPRBreadarrlinecb(XPRB_FGETS, datafile, 99, "g,", IFVAN[s], NCust);
 fclose(datafile);
}

/***********************************************************************/

int main(int argc, char **argv)
{
 XPRBprob prob;

 prob=XPRBnewprob("Delivery");   /* Initialize a new problem in BCL */
 readdata();                     /* Data input from file */
 moddelivery(prob);              /* Problem formulation & solving */

 return 0;
}


